# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
import sys
import json
import os
import requests
import six

session = requests.Session()


class DotDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        super(DotDict, self).__init__()
        for key, value in dct.items():
            if hasattr(value, 'keys'):
                value = DotDict(value)
            self[key] = value

    def __delitem__(self, key):
        super(DotDict, self).__delitem__(key)
        del self.__dict__[key]

    # setstate and getstate is for pickle
    def __getstate__(self):
        pass

    def __setstate__(self, *args, **kwargs):
        pass


def _load_json(str):
    if sys.version_info.major >= 3:
        return json.loads(str)

    def json_loads_byteified(json_text):
        return _byteify(
            json.loads(json_text, object_hook=_byteify),
            ignore_dicts=True
        )

    def _byteify(data, ignore_dicts=False):
        if isinstance(data, unicode):
            return data.encode('utf-8')
        if isinstance(data, list):
            return [_byteify(item, ignore_dicts=True) for item in data]
        if isinstance(data, dict) and not ignore_dicts:
            return {
                _byteify(key, ignore_dicts=True): _byteify(value, ignore_dicts=True)
                for key, value in data.iteritems()
            }
        return data

    return json_loads_byteified(str)


_parameter = None


def get_parameter():
    global _parameter
    if _parameter is not None:
        return _parameter
    parameter = os.getenv('AUTOCNN_PARAMETER', None)
    try:
        param = DotDict(_load_json(parameter)) if parameter else None
        _parameter = param
        return _parameter
    except Exception as e:
        print('Could get parameters. {}'.format(e))
        return None


def write_parameter():
    argv = sys.argv
    if argv[1] == 'write' and len(argv) > 2:
        output_yml = argv[2]
    else:
        print('Write Failed !!!\nUsage `helper write yml_file_name` to write parameters to yml')
        sys.exit(1)
    parameter = os.getenv('AUTOCNN_PARAMETER', None)
    try:
        if parameter is None:
            raise ValueError('Parameter is None')
        path = os.path.dirname(output_yml)
        if path and not os.path.isdir(path):
            os.makedirs(path)
        with open(output_yml, 'w') as fo:
            fo.write(parameter)
    except Exception as e:
        print('Could get parameters. {}'.format(e))


def get_exp_info():
    """Returns :
        * project_name
        * exp_name
        * project_uuid
        * exp_uuid
    """
    info = os.getenv('AUTOCNN_EXP_INFO', None)
    try:
        return json.loads(info) if info else None
    except (ValueError, TypeError):
        print('Could get exp info, '
              'please make sure this is running inside a autodl job.')
        return None


def get_api(version='v1'):
    api = os.getenv('AUTOCNN_API', None)
    if not api:
        print('Could get api info, '
              'please make sure this is running inside a autodl job.')
        return None
    return '{}/api/{}'.format(api, version)


def get_user_token():
    return os.getenv('AUTOCNN_USER_TOKEN', None)


def send_metrics(**metrics):
    exp_info = get_exp_info()
    exp_name = exp_info.get('exp_name', None)
    user_token = get_user_token()
    api = get_api()
    if not all([exp_name, user_token, api]):
        print('Environment information not found, '
              'please make sure this is running inside a autodl job.')
        return

    values = exp_name.split('.')
    user, project, exp = values[0], values[1], values[-1]

    try:
        formatted_metrics = {k: float(v) for k, v in six.iteritems(metrics)}
    except (ValueError, TypeError):
        print('Could not send metrics {}'.format(metrics))
        return

    try:
        session.post('{}/exp/{}/{}/{}/metrics/'.format(api, user, project, exp),
                     headers={"Authorization": "token {}".format(user_token)},
                     data={'values': json.dumps(formatted_metrics)},
                     timeout=0.1)
    except requests.RequestException as e:
        print('Could not reach autodl api {}'.format(e))



# from __future__ import unicode_literals
# import json
# import sys
# import multiprocessing
#
# import gunicorn.app.base
# from gunicorn.six import iteritems
#
# from flask import Flask, request, abort, Response
#
# app = Flask(__name__)
#
#
# class Mapper:
#     __mapper_relation = {}
#
#     @staticmethod
#     def register(cls, value):
#         Mapper.__mapper_relation[cls] = value
#
#     @staticmethod
#     def exist(cls):
#         if cls in Mapper.__mapper_relation:
#             return True
#         return False
#
#     @staticmethod
#     def get_value(cls):
#         return Mapper.__mapper_relation[cls]
#
#
# class AutoFill(type):
#     def __call__(cls, *args, **kwargs):
#         obj = cls.__new__(cls, *args, **kwargs)
#         arg_list = list(args)
#         if Mapper.exist(cls):
#             value = Mapper.get_value(cls)
#             arg_list.append(value)
#         obj.__init__(*arg_list, **kwargs)
#         return obj
#
#
# class StandaloneApplication(gunicorn.app.base.BaseApplication):
#
#     def __init__(self, app, options=None):
#         self.options = options or {}
#         self.application = app
#         super(StandaloneApplication, self).__init__()
#
#     def load_config(self):
#         config = dict([(key, value) for key, value in iteritems(self.options)
#                        if key in self.cfg.settings and value is not None])
#         for key, value in iteritems(config):
#             self.cfg.set(key.lower(), value)
#
#     def load(self):
#         return self.application
#
#
# class Deploy(metaclass=AutoFill):
#
#     def __init__(self, f=None):
#         self.f = f
#
#     @staticmethod
#     def register(func):
#         Mapper.register(Deploy, func)
#
#     @classmethod
#     def run(cls):
#         options = {
#             'bind': '%s:%s' % ('127.0.0.1', '5050'),
#             'workers': cls.num_of_workers(),
#         }
#         StandaloneApplication(app, options).run()
#
#     @classmethod
#     def num_of_workers(cls):
#         workers = int(sys.argv[1]) if len(sys.argv) > 1 or int(sys.argv[1]) < 1 else 1
#         assert workers > 0, ValueError
#         return workers if workers < multiprocessing.cpu_count() * 2 else multiprocessing.cpu_count() * 2
#
#
# @app.route('/_health', methods=['GET'])
# def health():
#     return Response('ok')
#
#
# @app.route('/', methods=['GET', 'POST'])
# def infer():
#     if request.values.get('image_numpy'):
#         d = Deploy()
#         return json.dumps(dict(
#             code=200,
#             data=d.f(request.values.get('image_numpy'))
#         ))
#     else:
#         abort(404)
#
#
# @app.errorhandler(404)
# def page_not_found(error):
#     return json.dumps(dict(
#         code=404,
#         err_msg='Not found, {}'.format(error)
#     ))
#



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(name='autodl-helper',
      version='0.0.4',
      license='MIT',
      platforms='any',
      packages=find_packages(),
      install_requires=[
          'requests==2.18.4',
      ],
      entry_points={
          "console_scripts": [
              "helper = autodel_helper.config:write_parameter",
          ],
      },
      classifiers=[
          'Programming Language :: Python',
          'Operating System :: OS Independent',
      ])
